@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.88 2005/07/04 12:22:40 chpe Exp $
+ *  $Id: downloader-view.c,v 1.92 2006/02/02 17:27:40 chpe Exp $
  */
 
 #include "config.h" 
@@ -70,7 +70,7 @@ enum
 
 #define EPHY_DOWNLOADER_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_DOWNLOADER_VIEW, DownloaderViewPrivate))
 
-struct DownloaderViewPrivate
+struct _DownloaderViewPrivate
 {
 	GtkTreeModel *model;
 	GHashTable *downloads_hash;
@@ -105,6 +105,12 @@ EphyDialogProperty properties [] =
 	{ NULL }
 };
 
+enum
+{
+	RESPONSE_PAUSE = 1,
+	RESPONSE_STOP = 2
+};
+
 static void
 downloader_view_build_ui (DownloaderView *dv);
 static void
@@ -113,15 +119,14 @@ static void
 downloader_view_finalize (GObject *object);
 static void
 downloader_view_init (DownloaderView *dv);
-
-/* Callbacks */
-void
-download_dialog_pause_cb (GtkButton *button, DownloaderView *dv);
-void
-download_dialog_abort_cb (GtkButton *button, DownloaderView *dv);
-gboolean
-download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
-			   DownloaderView *dv);
+static void
+download_dialog_response_cb (GtkWidget *dialog,
+			     int response,
+			     DownloaderView *dv);
+static gboolean
+download_dialog_delete_event_cb (GtkWidget *window,
+				 GdkEventAny *event,
+				 DownloaderView *dv);
 
 static GObjectClass *parent_class = NULL;
 
@@ -181,7 +186,7 @@ status_icon_popup_menu_cb (EggStatusIcon *icon,
 	menu = gtk_menu_new ();
 
 	/* this opens the downloader window, or brings it to the foreground if already open */
-	item = gtk_menu_item_new_with_mnemonic (_("_Show Downloader..."));
+	item = gtk_menu_item_new_with_mnemonic (_("_Show Downloads"));
 	g_signal_connect_swapped (item, "activate",
 				  G_CALLBACK (show_downloader_cb), dv);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -365,7 +370,8 @@ update_buttons (DownloaderView *dv)
 	}
 	gtk_widget_set_sensitive (dv->priv->pause_button, pause_enabled);
 	gtk_widget_set_sensitive (dv->priv->abort_button, abort_enabled);
-	gtk_button_set_label (GTK_BUTTON (dv->priv->pause_button), label_pause ? _("_Pause") : _("_Resume"));
+	gtk_button_set_label (GTK_BUTTON (dv->priv->pause_button),
+			      label_pause ? _("_Pause") : _("_Resume"));
 }
 
 static void
@@ -406,7 +412,7 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 
 	name = ephy_download_get_name (download);
 
-	if (total != -1)
+	if (total != -1 && current != -1)
 	{
 		char *total_progress;
 
@@ -416,10 +422,14 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 					cur_progress, total_progress);
 		g_free (total_progress);
 	}
-	else
+	else if (current != -1)
 	{
 		file = g_strdup_printf ("%s\n%s", name, cur_progress);
 	}
+	else
+	{
+		file = g_strdup_printf ("%s\n%s", name, _("Unknown"));
+	}
 
 	if (remaining_secs < 0)
 	{
@@ -621,6 +631,11 @@ downloader_view_build_ui (DownloaderView *dv)
 		 properties[PROP_ABORT_BUTTON].id, &priv->abort_button,
 		 NULL);
 
+	g_signal_connect (priv->window, "response",
+			  G_CALLBACK (download_dialog_response_cb), dv);
+	g_signal_connect (priv->window, "delete-event",
+			  G_CALLBACK (download_dialog_delete_event_cb), dv);
+
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)),
 				     GTK_SELECTION_BROWSE);
 
@@ -691,8 +706,8 @@ downloader_view_build_ui (DownloaderView *dv)
 	g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), dv);
 }
 
-void
-download_dialog_pause_cb (GtkButton *button, DownloaderView *dv)
+static void
+download_dialog_pause (DownloaderView *dv)
 {
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
@@ -790,8 +805,8 @@ downloader_view_remove_download (DownloaderView *dv, EphyDownload *download)
 	}
 }
 
-void
-download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
+static void
+download_dialog_stop (DownloaderView *dv)
 {
 	GValue val = {0, };
 	GtkTreeSelection *selection;
@@ -814,9 +829,25 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
 	g_value_unset (&val);
 }
 
-gboolean
-download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
-			   DownloaderView *dv)
+static void
+download_dialog_response_cb (GtkWidget *dialog,
+			     int response,
+			     DownloaderView *dv)
+{
+	if (response == RESPONSE_PAUSE)
+	{
+		download_dialog_pause (dv);
+	}
+	else if (response == RESPONSE_STOP)
+	{
+		download_dialog_stop (dv);
+	}
+}
+
+static gboolean
+download_dialog_delete_event_cb (GtkWidget *window,
+				 GdkEventAny *event,
+				 DownloaderView *dv)
 {
 	/* FIXME multi-head */
 	if (egg_tray_manager_check_running (gdk_screen_get_default ()))
