@@ -1,7 +1,7 @@
 /*  vim:set ts=8 noet sw=8:
  *  Copyright (C) 2000-2004 Marco Pesenti Gritti
  *  Copyright (C) 2003 Robert Marcano
- *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2003, 2004, 2005, 2006 Christian Persch
  *  Copyright (C) 2005 Crispin Flowerday
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-single.cpp,v 1.122 2005/12/15 10:50:22 jframeau Exp $
+ *  $Id: mozilla-embed-single.cpp,v 1.137 2006/02/10 19:01:18 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -36,6 +36,7 @@
 #include "glib.h"
 #include "ephy-debug.h"
 #include "gtkmozembed.h"
+#include "gtkmozembed_internal.h"
 #include "mozilla-embed.h"
 #include "ephy-file-helpers.h"
 #include "mozilla-notifiers.h"
@@ -69,9 +70,14 @@
 #include <nsCPasswordManager.h>
 #include <nsIPermission.h>
 #include <nsIPermissionManager.h>
+#include <nsIFile.h>
 #include <nsILocalFile.h>
 #include <nsIURI.h>
 
+#if defined(HAVE_MOZILLA_TOOLKIT) && defined(HAVE_GECKO_1_8)
+#include "EphyDirectoryProvider.h"
+#endif
+
 #ifdef HAVE_MOZILLA_PSM
 #include <nsIX509Cert.h>
 #include <nsIX509CertDB.h>
@@ -99,21 +105,39 @@
 
 #include <stdlib.h>
 
+#ifdef HAVE_GECKO_1_8
+#include <nsIURI.h>
+#include <nsIStyleSheetService.h>
+#include "EphyUtils.h"
+#include "ephy-file-helpers.h"
+#endif
+
 #define MOZILLA_PROFILE_DIR  "/mozilla"
 #define MOZILLA_PROFILE_NAME "epiphany"
 #define MOZILLA_PROFILE_FILE "prefs.js"
 #define DEFAULT_PROFILE_FILE SHARE_DIR"/default-prefs.js"
 
+#define USER_CSS_LOAD_DELAY 500 /* ms */
+
 #define MOZILLA_EMBED_SINGLE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED_SINGLE, MozillaEmbedSinglePrivate))
 
 struct MozillaEmbedSinglePrivate
 {
 	char *user_prefs;
 	
+	EphySingle *mSingleObserver;
+
+#ifndef HAVE_GECKO_1_8
 	/* monitor this widget for theme changes*/
 	GtkWidget *theme_window;
+#endif
 
-	EphySingle *mSingleObserver;
+#ifdef HAVE_GECKO_1_8
+	char *user_css_file;
+        guint user_css_enabled_notifier_id;
+        EphyFileMonitor *user_css_file_monitor;
+        guint user_css_enabled : 1;
+#endif
 
 	guint online : 1;
 };
@@ -273,22 +297,23 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 
 #ifdef HAVE_GECKO_1_8
 	/* FIXME: maybe only set the major version ("1.6", "1.8") here? */
-	pref->SetCharPref ("general.useragent.extra.epiphany", "Epiphany/" VERSION);
+	pref->SetCharPref ("general.useragent.extra.epiphany", "Epiphany/" EPIPHANY_MAJOR);
 
 	/* Unset old prefs, otherwise they end up in the user agent string too */
 	pref->ClearUserPref ("general.useragent.vendor");
 	pref->ClearUserPref ("general.useragent.vendorSub");
 #else
 	pref->SetCharPref ("general.useragent.vendor", "Epiphany");
-	pref->SetCharPref ("general.useragent.vendorSub", VERSION);
+	pref->SetCharPref ("general.useragent.vendorSub", EPIPHANY_MAJOR);
 #endif
 
-	/* Open ftp uris with an external handler if one is setup */
+	/* Don't open ftp uris with an external handler if one is setup */
 	pref->SetBoolPref ("network.protocol-handler.external.ftp", PR_FALSE);	
 
 	return TRUE;
 }
 
+#ifndef HAVE_GECKO_1_8
 static char *
 color_to_string (GdkColor color)
 {
@@ -351,6 +376,7 @@ mozilla_setup_colors (MozillaEmbedSingle *mes)
 
 	mes->priv->theme_window = window;
 }			
+#endif
 
 static void 
 mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *moz_single,
@@ -420,21 +446,12 @@ mozilla_init_single (MozillaEmbedSingle *mes)
 			  mes);
 }
 
-static void
-mozilla_init_home (void)
-{
-	char *mozilla_five_home;
-        mozilla_five_home = g_strdup (g_getenv ("MOZILLA_FIVE_HOME"));
-        gtk_moz_embed_set_comp_path (mozilla_five_home);
-        g_free (mozilla_five_home);
-}
-
 void
 mozilla_init_profile (void)
 {
 	char *profile_path;
 	profile_path = g_build_filename (ephy_dot_dir (), 
-					 MOZILLA_PROFILE_DIR, 
+					 MOZILLA_PROFILE_DIR,
 					 NULL);
         gtk_moz_embed_set_profile_path (profile_path, MOZILLA_PROFILE_NAME);
         g_free (profile_path);
@@ -517,19 +534,219 @@ mozilla_init_observer (MozillaEmbedSingle *single)
 	}
 }
 
-static gboolean
-init_services (MozillaEmbedSingle *single)
+#ifdef HAVE_GECKO_1_8
+
+static void
+user_css_register (MozillaEmbedSingle *single)
 {
+	MozillaEmbedSinglePrivate *priv = single->priv;
+
+	nsresult rv;
+	nsCOMPtr<nsILocalFile> file;
+	rv = NS_NewNativeLocalFile (nsDependentCString (priv->user_css_file),
+				    PR_TRUE, getter_AddRefs (file));
+	NS_ENSURE_SUCCESS (rv, );
+
+	PRBool exists = PR_FALSE;
+	rv = file->Exists (&exists);
+	if (NS_FAILED (rv) || !exists) return;
+
+	nsCOMPtr<nsIURI> uri;
+	rv = EphyUtils::NewFileURI (getter_AddRefs (uri), file);
+	NS_ENSURE_SUCCESS (rv, );
+
+	nsCOMPtr<nsIStyleSheetService> service
+			(do_GetService ("@mozilla.org/content/style-sheet-service;1", &rv));
+	NS_ENSURE_SUCCESS (rv, );
+
+	PRBool isRegistered = PR_FALSE;
+	rv = service->SheetRegistered (uri, nsIStyleSheetService::USER_SHEET,
+				       &isRegistered);
+	if (NS_SUCCEEDED (rv) && isRegistered)
+	{
+		rv = service->UnregisterSheet (uri, nsIStyleSheetService::USER_SHEET);
+	}
+
+	rv = service->LoadAndRegisterSheet (uri, nsIStyleSheetService::AGENT_SHEET);
+	if (NS_FAILED (rv))
+	{
+		g_warning ("Registering the user stylesheet failed (rv=%x)!\n", rv);
+	}
+}
+
+static void
+user_css_unregister (MozillaEmbedSingle *single)
+{
+	MozillaEmbedSinglePrivate *priv = single->priv;
+
+	nsresult rv;
+	nsCOMPtr<nsILocalFile> file;
+	rv = NS_NewNativeLocalFile (nsDependentCString (priv->user_css_file),
+				    PR_TRUE, getter_AddRefs (file));
+	NS_ENSURE_SUCCESS (rv, );
+
+	nsCOMPtr<nsIURI> uri;
+	rv = EphyUtils::NewFileURI (getter_AddRefs (uri), file);
+	NS_ENSURE_SUCCESS (rv, );
+
+	nsCOMPtr<nsIStyleSheetService> service
+			(do_GetService ("@mozilla.org/content/style-sheet-service;1", &rv));
+	NS_ENSURE_SUCCESS (rv, );
+
+	PRBool isRegistered = PR_FALSE;
+	rv = service->SheetRegistered (uri, nsIStyleSheetService::USER_SHEET,
+				       &isRegistered);
+	if (NS_SUCCEEDED (rv) && isRegistered)
+	{
+		rv = service->UnregisterSheet (uri, nsIStyleSheetService::USER_SHEET);
+	}
+	if (NS_FAILED (rv))
+	{
+		g_warning ("Unregistering the user stylesheet failed (rv=%x)!\n", rv);
+	}
+}
+
+static void
+user_css_file_monitor_func (EphyFileMonitor *,
+			    const char *,
+			    GnomeVFSMonitorEventType event_type,
+			    MozillaEmbedSingle *single)
+{
+	LOG ("Reregistering the user style sheet");
+
+	if (event_type == GNOME_VFS_MONITOR_EVENT_DELETED)
+	{
+		user_css_unregister (single);
+	}
+	else
+	{
+		user_css_register (single);
+	}
+}
+
+static void
+user_css_enabled_notify (GConfClient *client,
+			 guint cnxn_id,
+			 GConfEntry *entry,
+			 MozillaEmbedSingle *single)
+{
+	MozillaEmbedSinglePrivate *priv = single->priv;
+	guint enabled;
+
+	enabled = eel_gconf_get_boolean (CONF_USER_CSS_ENABLED) != FALSE;
+	if (priv->user_css_enabled == enabled) return;
+
+	LOG ("User stylesheet enabled: %s", enabled ? "t" : "f");
+
+	priv->user_css_enabled = enabled;
+
+        if (enabled)
+	{
+		char *uri;
+
+		user_css_register (single);
+
+		uri = gnome_vfs_get_uri_from_local_path (priv->user_css_file);
+
+		g_assert (priv->user_css_file_monitor == NULL);
+		priv->user_css_file_monitor =
+			ephy_file_monitor_add (uri,
+					       GNOME_VFS_MONITOR_FILE,
+					       USER_CSS_LOAD_DELAY,
+					       (EphyFileMonitorFunc) user_css_file_monitor_func,
+					       NULL,
+					       single);
+		g_free (uri);
+	}
+        else
+	{
+		if (priv->user_css_file_monitor != NULL)
+		{
+			ephy_file_monitor_cancel (priv->user_css_file_monitor);
+			priv->user_css_file_monitor = NULL;
+		}
+
+		user_css_unregister (single);
+	}
+}
+
+static void
+mozilla_stylesheet_init (MozillaEmbedSingle *single)
+{
+	MozillaEmbedSinglePrivate *priv = single->priv;
+
+	priv->user_css_file = g_build_filename (ephy_dot_dir (),
+						USER_STYLESHEET_FILENAME,
+						NULL);
+
+	user_css_enabled_notify (NULL, 0, NULL, single);
+	priv->user_css_enabled_notifier_id =
+		eel_gconf_notification_add
+			(CONF_USER_CSS_ENABLED,
+			 (GConfClientNotifyFunc) user_css_enabled_notify,
+			 single);
+}
+
+static void
+mozilla_stylesheet_shutdown (MozillaEmbedSingle *single)
+{
+	MozillaEmbedSinglePrivate *priv = single->priv;
+
+	if (priv->user_css_enabled_notifier_id != 0)
+	{
+		eel_gconf_notification_remove (priv->user_css_enabled_notifier_id);
+		priv->user_css_enabled_notifier_id = 0;
+	}
+
+	if (priv->user_css_file_monitor != NULL)
+	{
+		ephy_file_monitor_cancel (priv->user_css_file_monitor);
+		priv->user_css_file_monitor = NULL;
+	}
+
+	if (priv->user_css_file != NULL)
+	{
+		g_free (priv->user_css_file);
+		priv->user_css_file = NULL;
+	}
+}
+
+#endif /* HAVE_GECKO_1_8 */
+
+static gboolean
+impl_init (EphyEmbedSingle *esingle)
+{
+	MozillaEmbedSingle *single = MOZILLA_EMBED_SINGLE (esingle);
+
+#ifdef HAVE_GECKO_1_9
+	NS_LogInit ();
+#endif
+
 	/* Pre initialization */
 	mozilla_init_plugin_path ();
-	mozilla_init_home ();
+
 	mozilla_init_profile ();
 
+#ifdef HAVE_GECKO_1_9
+	gtk_moz_embed_set_path (MOZILLA_HOME);
+#else
 	/* Set mozilla binary path */
 	gtk_moz_embed_set_comp_path (MOZILLA_HOME);
+#endif
+
+#if defined(HAVE_MOZILLA_TOOLKIT) && defined(HAVE_GECKO_1_8)
+
+	nsCOMPtr<nsIDirectoryServiceProvider> dp = new EphyDirectoryProvider ();
+	if (!dp) return FALSE;
+
+	gtk_moz_embed_set_directory_service_provider (dp);
+#endif
 
 	/* Fire up the beast */
 	gtk_moz_embed_push_startup ();
+	/* FIXME check that it succeeded! */
+
+	mozilla_register_components ();
 
 	/* Until gtkmozembed does this itself */
 	mozilla_init_chrome ();
@@ -541,19 +758,23 @@ init_services (MozillaEmbedSingle *single)
 		return FALSE;
 	}
 
+#ifndef HAVE_GECKO_1_8
 	/* FIXME: This should be removed when mozilla
 	 * bugs 207000 and 207001 are fixed.
 	 */
 	mozilla_setup_colors (single);
+#endif
 
 	START_PROFILER ("Mozilla prefs notifiers")
 	mozilla_notifiers_init ();
 	STOP_PROFILER ("Mozilla prefs notifiers")
 
-	mozilla_register_components ();
-
 	mozilla_init_observer (single);
 
+#ifdef HAVE_GECKO_1_8
+        mozilla_stylesheet_init (single);
+#endif
+
 	return TRUE;
 }
 
@@ -583,22 +804,6 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 				  MOZILLA_PROFILE_FILE,
 				  NULL);
 
-	if (!init_services (mes))
-	{
-		GtkWidget *dialog;
-
-		dialog = gtk_message_dialog_new
-			(NULL,
-                         GTK_DIALOG_MODAL,
-                         GTK_MESSAGE_ERROR,
-                         GTK_BUTTONS_CLOSE,
-                         _("Epiphany can't be used now. "
-                         "Mozilla initialization failed."));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-
-		exit (0);
-	}
-
 	g_signal_connect_object (embed_shell, "prepare-close",
 				 G_CALLBACK (prepare_close_cb), mes,
 				 (GConnectFlags) 0);
@@ -608,12 +813,17 @@ static void
 mozilla_embed_single_dispose (GObject *object)
 {
 	MozillaEmbedSingle *single = MOZILLA_EMBED_SINGLE (object);
+	MozillaEmbedSinglePrivate *priv = single->priv;
 
-	if (single->priv->mSingleObserver)
+#ifdef HAVE_GECKO_1_8
+        mozilla_stylesheet_shutdown (single);
+#endif
+
+	if (priv->mSingleObserver)
 	{
-		single->priv->mSingleObserver->Detach ();
-		NS_RELEASE (single->priv->mSingleObserver);
-		single->priv->mSingleObserver = nsnull;
+		priv->mSingleObserver->Detach ();
+		NS_RELEASE (priv->mSingleObserver);
+		priv->mSingleObserver = nsnull;
 	}
 
 	parent_class->dispose (object);
@@ -632,12 +842,18 @@ mozilla_embed_single_finalize (GObject *object)
 
 	gtk_moz_embed_pop_startup ();
 
+#ifdef HAVE_GECKO_1_9
+	NS_LogTerm ();
+#endif
+
 	g_free (mes->priv->user_prefs);
 
+#ifndef HAVE_GECKO_1_8
 	if (mes->priv->theme_window)
 	{
 		gtk_widget_destroy (mes->priv->theme_window);
 	}
+#endif
 }
 
 static void
@@ -694,6 +910,22 @@ impl_get_network_status (EphyEmbedSingle *esingle)
 	return !isOffline;
 }
 
+static const char*
+impl_get_backend_name (EphyEmbedSingle *esingle)
+{
+	/* If you alter the return values here, remember to update
+	 * the docs in ephy-embed-single.c */
+#if defined (HAVE_GECKO_1_10)
+# error "Need to add version string for gecko 1.10"
+#elif defined(HAVE_GECKO_1_9)
+	return "gecko-1.9";
+#elif defined(HAVE_GECKO_1_8)
+	return "gecko-1.8";
+#elif defined(HAVE_GECKO_1_7)
+	return "gecko-1.7";
+#endif
+}
+
 static GList *
 impl_get_font_list (EphyEmbedSingle *shell,
 		    const char *langGroup)
@@ -1205,12 +1437,14 @@ mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 static void
 ephy_embed_single_iface_init (EphyEmbedSingleIface *iface)
 {
+	iface->init = impl_init;
 	iface->clear_cache = impl_clear_cache;
 	iface->clear_auth_cache = impl_clear_auth_cache;
 	iface->set_network_status = impl_set_network_status;
 	iface->get_network_status = impl_get_network_status;
 	iface->get_font_list = impl_get_font_list;
 	iface->open_window = impl_open_window;
+	iface->get_backend_name = impl_get_backend_name;
 }
 
 static void
