@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-gui.h,v 1.21 2006/01/29 21:47:21 chpe Exp $
+ *  $Id: ephy-gui.h,v 1.23 2006/02/13 23:02:10 chpe Exp $
  */
 
 #ifndef EPHY_GUI_H
@@ -77,8 +77,7 @@ void		ephy_gui_help				 (GtkWindow *parent,
 void		ephy_gui_window_update_user_time	 (GtkWidget *window,
 							  guint32 user_time);
 
-void		ephy_gui_window_present			 (GtkWindow *window,
-							  guint32 user_time);
+GtkWidget      *ephy_gui_message_dialog_get_content_box	 (GtkWidget *dialog);
 
 G_END_DECLS
 
