@@ -17,20 +17,21 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmark-action.c,v 1.70 2005/11/08 12:56:24 chpe Exp $
+ *  $Id: ephy-bookmark-action.c,v 1.77 2006/02/10 22:49:29 chpe Exp $
  */
 
 #include "config.h"
 
 #include "egg-editable-toolbar.h"
 #include "ephy-bookmark-action.h"
+#include "ephy-bookmarks-ui.h"
 #include "ephy-bookmarks.h"
-#include "ephy-link-action.h"
-#include "ephy-link.h"
+#include "ephy-stock-icons.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-shell.h"
 #include "ephy-gui.h"
 #include "ephy-debug.h"
+#include "ephy-dnd.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtkwidget.h>
@@ -49,14 +50,15 @@
 
 #include <string.h>
 
+static const GtkTargetEntry drag_types[] = {
+  {EPHY_DND_URL_TYPE, 0, 0},
+};
+
 /* FIXME tweak this, or make it configurable? (bug 148093) */
 #define ENTRY_WIDTH_CHARS	12
 #define TOOLITEM_WIDTH_CHARS	20
 #define LABEL_WIDTH_CHARS       32
 
-static void ephy_bookmark_action_init       (EphyBookmarkAction *action);
-static void ephy_bookmark_action_class_init (EphyBookmarkActionClass *class);
-
 #define EPHY_BOOKMARK_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARK_ACTION, EphyBookmarkActionPrivate))
 
 struct _EphyBookmarkActionPrivate
@@ -76,35 +78,7 @@ enum
 	PROP_ICON
 };
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_bookmark_action_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		static const GTypeInfo type_info =
-		{
-			sizeof (EphyBookmarkActionClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) ephy_bookmark_action_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (EphyBookmarkAction),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_bookmark_action_init,
-		};
-
-		type = g_type_register_static (EPHY_TYPE_LINK_ACTION,
-					       "EphyBookmarkAction",
-					       &type_info, 0);
-	}
-
-	return type;
-}
+static GObjectClass *parent_class;
 
 static GtkWidget *
 create_tool_item (GtkAction *action)
@@ -126,7 +100,6 @@ create_tool_item (GtkAction *action)
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (hbox), button);
 	g_object_set_data (G_OBJECT (item), "button", button);
-	g_object_set_data (G_OBJECT (item), "egg-drag-source", button);
 
 	entry = gtk_entry_new ();
 	gtk_entry_set_width_chars (GTK_ENTRY (entry), ENTRY_WIDTH_CHARS);
@@ -153,13 +126,17 @@ create_tool_item (GtkAction *action)
 }
 
 static void
-ephy_bookmark_action_sync_smart_url (GtkAction *gaction, GParamSpec *pspec, GtkWidget *proxy)
+ephy_bookmark_action_sync_smart_url (GtkAction *gaction,
+				     GParamSpec *pspec,
+				     GtkWidget *proxy)
 {
 	if (GTK_IS_TOOL_ITEM (proxy))
 	{
 		EphyBookmarkAction *action = EPHY_BOOKMARK_ACTION (gaction);
 		EphyBookmarkActionPrivate *priv = action->priv;
 		gboolean is_smart_url = priv->smart_url;
+		gboolean has_icon = (gboolean) ephy_node_get_property_string
+		  (priv->node, EPHY_NODE_BMK_PROP_ICON);		
 		GtkWidget *entry, *icon;
 		guint width;
 
@@ -168,8 +145,8 @@ ephy_bookmark_action_sync_smart_url (GtkAction *gaction, GParamSpec *pspec, GtkW
 		entry = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "entry"));
 		icon = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "icon"));
 
-		g_object_set (icon, "visible", !is_smart_url, NULL);
 		g_object_set (entry, "visible", is_smart_url, NULL);
+		g_object_set (icon, "visible", !is_smart_url || has_icon, NULL);
 		gtk_entry_set_width_chars (GTK_ENTRY (entry),
 					   width > 0 ? width : ENTRY_WIDTH_CHARS);
 	}
@@ -197,7 +174,9 @@ favicon_cache_changed_cb (EphyFaviconCache *cache,
 }
 
 static void
-ephy_bookmark_action_sync_icon (GtkAction *action, GParamSpec *pspec, GtkWidget *proxy)
+ephy_bookmark_action_sync_icon (GtkAction *action,
+				GParamSpec *pspec,
+				GtkWidget *proxy)
 {
 	EphyBookmarkAction *bma = EPHY_BOOKMARK_ACTION (action);
 	const char *icon_location;
@@ -210,9 +189,9 @@ ephy_bookmark_action_sync_icon (GtkAction *action, GParamSpec *pspec, GtkWidget
 						       EPHY_NODE_BMK_PROP_ICON);
 
 	cache = EPHY_FAVICON_CACHE (ephy_embed_shell_get_favicon_cache
-		(EPHY_EMBED_SHELL (ephy_shell)));
+		(ephy_embed_shell_get_default ()));
 
-	if (icon_location)
+	if (icon_location && *icon_location)
 	{
 		pixbuf = ephy_favicon_cache_get (cache, icon_location);
 
@@ -233,9 +212,9 @@ ephy_bookmark_action_sync_icon (GtkAction *action, GParamSpec *pspec, GtkWidget
 		icon = GTK_IMAGE (g_object_get_data (G_OBJECT (proxy), "icon"));
 		g_return_if_fail (icon != NULL);
 
-		if (pixbuf == NULL)
+		if (pixbuf == NULL && icon_location == NULL)
 		{
-			pixbuf = gtk_widget_render_icon (proxy, GTK_STOCK_NEW,
+			pixbuf = gtk_widget_render_icon (proxy, STOCK_BOOKMARK,
 							 GTK_ICON_SIZE_MENU, NULL);
 		}
 
@@ -293,10 +272,10 @@ ephy_bookmark_action_sync_label (GtkAction *gaction,
 	}
 }
 
-static void
-bookmark_activate_with_flags (GtkWidget *widget,
-				EphyBookmarkAction *action,
-				EphyLinkFlags flags)
+void
+ephy_bookmark_action_activate (EphyBookmarkAction *action,
+			       GtkWidget *widget,
+			       EphyLinkFlags flags)
 {
 	EphyBookmarkActionPrivate *priv = action->priv;
 	EphyBookmarks *bookmarks;
@@ -343,98 +322,48 @@ bookmark_activate_with_flags (GtkWidget *widget,
 	g_free (text);
 }
 
-static void
-open_in_tab_activate_cb (GtkWidget *widget,
-			 EphyBookmarkAction *action)
-{
-	bookmark_activate_with_flags(widget, action,
-			EPHY_LINK_NEW_TAB | EPHY_LINK_JUMP_TO);
-}
-
-static void
-open_in_window_activate_cb (GtkWidget *widget,
-			EphyBookmarkAction *action)
-{
-	bookmark_activate_with_flags(widget, action,
-			EPHY_LINK_NEW_WINDOW);
-}
-
 static void
 activate_cb (GtkWidget *widget,
-	     	EphyBookmarkAction *action)
+	     EphyBookmarkAction *action)
 {
-	bookmark_activate_with_flags(widget, action,
-			ephy_gui_is_middle_click () ? EPHY_LINK_NEW_TAB : 0);
-}
+	gboolean control = FALSE;
+	GdkEvent *event;
 
-static void
-properties_activate_cb (GtkWidget *menu,
-			EphyBookmarkAction *action)
-{
-	GtkWidget *window, *proxy;
-	EphyBookmarks *bookmarks;
-
-	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	proxy = g_object_get_data (G_OBJECT (menu), "proxy");
-	window = gtk_widget_get_toplevel (proxy);
-
-	ephy_bookmarks_show_bookmark_properties
-		(bookmarks, action->priv->node, window);
-}
-
-static void
-show_context_menu (EphyBookmarkAction *action,
-		   GtkWidget *proxy,
-		   GdkEventButton *event,
-		   GtkMenuPositionFunc func)
-{
-	GtkWidget *menu, *item;
-
-	menu = gtk_menu_new ();
-
-	item = gtk_menu_item_new_with_mnemonic (_("Open in New _Tab"));
-	gtk_widget_show (item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	g_signal_connect (item, "activate",
-			  G_CALLBACK (open_in_tab_activate_cb), action);
-
-	item = gtk_menu_item_new_with_mnemonic (_("Open in New _Window"));
-	gtk_widget_show (item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	g_signal_connect (item, "activate",
-			  G_CALLBACK (open_in_window_activate_cb), action);
-
-	item = gtk_separator_menu_item_new ();
-	gtk_widget_show (item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
-	g_object_set_data (G_OBJECT (item), "proxy", proxy);
-	gtk_widget_show (item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	g_signal_connect (item, "activate",
-			  G_CALLBACK (properties_activate_cb), action);
-    
-	egg_editable_toolbar_add_popup_items (proxy, GTK_MENU (menu));
-
-	if (event != NULL)
+	event = gtk_get_current_event ();
+	if (event)
 	{
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, func, proxy,
-				event->button, event->time);
-	}
-	else
-	{
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, func, proxy, 0,
-				gtk_get_current_event_time ());
-		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
+		if (event->type == GDK_KEY_PRESS ||
+		    event->type == GDK_KEY_RELEASE)
+		{
+			control = (event->key.state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK;
+		}
+			
+		gdk_event_free (event);
 	}
+
+	ephy_bookmark_action_activate
+	  (action, widget, (control || ephy_gui_is_middle_click ()) ? EPHY_LINK_NEW_TAB : 0);
 }
 
 static gboolean
-popup_menu_cb (GtkWidget *widget, EphyBookmarkAction *action)
+entry_key_press_cb (GtkEntry *entry,
+		    GdkEventKey *event,
+		    EphyBookmarkAction *action)
 {
-	show_context_menu (action, widget, NULL, ephy_gui_menu_position_under_widget);
-	return TRUE;
+	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
+
+	if ((event->keyval == GDK_Return ||
+	     event->keyval == GDK_KP_Enter ||
+	     event->keyval == GDK_ISO_Enter) &&
+	    state == GDK_CONTROL_MASK)
+	{
+		gtk_im_context_reset (entry->im_context);
+
+		g_signal_emit_by_name (entry, "activate");
+
+		return TRUE;
+	}
+	return FALSE;
 }
 
 static gboolean
@@ -442,12 +371,7 @@ button_press_cb (GtkWidget *widget,
 		 GdkEventButton *event,
 		 EphyBookmarkAction *action)
 {
-	if (event->button == 3)
-	{
-		show_context_menu (action, widget, event, NULL);
-		return TRUE;
-	}
-	else if (event->button == 2)	
+	if (event->button == 2)	
 	{
 		gtk_button_pressed (GTK_BUTTON (widget));
 	}
@@ -468,9 +392,25 @@ button_release_cb (GtkWidget *widget,
 	return FALSE;
 }
 
+static void
+drag_data_get_cb (GtkWidget          *widget,
+		  GdkDragContext     *context,
+		  GtkSelectionData   *selection_data,
+		  guint               info,
+		  guint32             time,
+		  GtkAction          *action)
+{
+	EphyNode *node = ephy_bookmark_action_get_bookmark (EPHY_BOOKMARK_ACTION (action));
+	const char *location = ephy_node_get_property_string (node, EPHY_NODE_BMK_PROP_LOCATION);
+
+	g_return_if_fail (location != NULL);
+
+	gtk_selection_data_set (selection_data, selection_data->target, 8, (unsigned char *)location, strlen (location));
+}
 
 static void
-connect_proxy (GtkAction *action, GtkWidget *proxy)
+connect_proxy (GtkAction *action,
+	       GtkWidget *proxy)
 {
 	GtkWidget *button, *entry;
 
@@ -494,8 +434,6 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 
 		button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
 		g_signal_connect (button, "clicked", G_CALLBACK (activate_cb), action);
-		g_signal_connect (button, "popup_menu",
-				  G_CALLBACK (popup_menu_cb), action);
 		g_signal_connect (button, "button-press-event",
 				  G_CALLBACK (button_press_cb), action);
 		g_signal_connect (button, "button-release-event",
@@ -503,6 +441,12 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 
 		entry = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "entry"));
 		g_signal_connect (entry, "activate", G_CALLBACK (activate_cb), action);
+		g_signal_connect (entry, "key-press-event", G_CALLBACK (entry_key_press_cb), action);
+
+		g_signal_connect (button, "drag-data-get",
+				  G_CALLBACK (drag_data_get_cb), action);
+		gtk_drag_source_set (button, GDK_BUTTON1_MASK, drag_types,
+				     G_N_ELEMENTS (drag_types), GDK_ACTION_COPY);
 	}
 	else if (GTK_IS_MENU_ITEM (proxy))
 	{
@@ -560,17 +504,20 @@ ephy_bookmark_action_get_bookmark (EphyBookmarkAction *action)
 void
 ephy_bookmark_action_set_bookmark (EphyBookmarkAction *action,
 				   EphyNode *node)
-{        
+{
+	EphyBookmarkActionPrivate *priv = action->priv;
+	GObject *object = G_OBJECT (action);
+
 	g_return_if_fail (node != NULL);
-	
-	action->priv->node = node;
-	
-	g_object_freeze_notify (G_OBJECT (action));
-	
-	g_object_notify (G_OBJECT (action), "bookmark");
+
+	priv->node = node;
+
+	g_object_freeze_notify (object);
+
+	g_object_notify (object, "bookmark");
 	ephy_bookmark_action_updated (action);
-	
-	g_object_thaw_notify (G_OBJECT (action));        
+
+	g_object_thaw_notify (object);
 }
 
 static void
@@ -602,37 +549,57 @@ ephy_bookmark_action_get_property (GObject *object,
 				   GParamSpec *pspec)
 {
 	EphyBookmarkAction *action = EPHY_BOOKMARK_ACTION (object);
+	EphyBookmarkActionPrivate *priv = action->priv;
 
-	g_return_if_fail (action->priv->node != NULL);
+	g_return_if_fail (priv->node != NULL);
 
 	switch (prop_id)
 	{
 		case PROP_BOOKMARK:
-			g_value_set_pointer (value, action->priv->node);
+			g_value_set_pointer (value, priv->node);
 			break;
 		case PROP_TOOLTIP:
 		case PROP_LOCATION:
 			g_value_set_string (value,
-				ephy_node_get_property_string (action->priv->node,
+				ephy_node_get_property_string (priv->node,
 					EPHY_NODE_BMK_PROP_LOCATION));
 			break;
 		case PROP_SMART_URL:
-			g_value_set_boolean (value, action->priv->smart_url);
+			g_value_set_boolean (value, priv->smart_url);
 			break;
 		case PROP_ICON:
 			g_value_set_string (value,
-				ephy_node_get_property_string (action->priv->node,
+				ephy_node_get_property_string (priv->node,
 					EPHY_NODE_BMK_PROP_ICON));
 			break;
 	}
 }
 
 static void
-ephy_bookmark_action_finalize (GObject *object)
+ephy_bookmark_action_init (EphyBookmarkAction *action)
 {
-	LOG ("Bookmark action %p finalized", object);
+	action->priv = EPHY_BOOKMARK_ACTION_GET_PRIVATE (action);
+	
+	action->priv->cache_handler = 0;
+}
 
-	parent_class->finalize (object);
+static void
+ephy_bookmark_action_dispose (GObject *object)
+{
+	EphyBookmarkAction *action = (EphyBookmarkAction *) object;
+	EphyBookmarkActionPrivate *priv = action->priv;
+	GObject *cache;
+
+	if (priv->cache_handler != 0)
+	{
+		cache = ephy_embed_shell_get_favicon_cache
+				(ephy_embed_shell_get_default ());
+
+		g_signal_handler_disconnect (cache, priv->cache_handler);
+		priv->cache_handler = 0;
+	}
+
+	parent_class->dispose (object);
 }
 
 static void
@@ -648,15 +615,15 @@ ephy_bookmark_action_class_init (EphyBookmarkActionClass *class)
 	action_class->menu_item_type = GTK_TYPE_IMAGE_MENU_ITEM;
 	action_class->connect_proxy = connect_proxy;
 
-	object_class->finalize = ephy_bookmark_action_finalize;
+	object_class->dispose = ephy_bookmark_action_dispose;
 	object_class->set_property = ephy_bookmark_action_set_property;
 	object_class->get_property = ephy_bookmark_action_get_property;
 
 	g_object_class_install_property (object_class,
 					 PROP_BOOKMARK,
 					 g_param_spec_pointer ("bookmark",
-							       "Bookmark",
-							       "Bookmark",
+							       "bookmark",
+							       "bookmark",
 							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
 							       G_PARAM_CONSTRUCT_ONLY));
 
@@ -664,61 +631,72 @@ ephy_bookmark_action_class_init (EphyBookmarkActionClass *class)
 	g_object_class_install_property (object_class,
 					 PROP_TOOLTIP,
 					 g_param_spec_string  ("tooltip",
-							       "Tooltip",
-							       "Tooltip",
+							       "tooltip",
+							       "tooltip",
 							       NULL,
 							       G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	
 	g_object_class_install_property (object_class,
 					 PROP_LOCATION,
 					 g_param_spec_string  ("location",
-							       "Location",
-							       "Location",
+							       "location",
+							       "location",
 							       NULL,
 							       G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	g_object_class_install_property (object_class,
 					 PROP_SMART_URL,
 					 g_param_spec_boolean  ("smarturl",
-								"Smart url",
-								"Smart url",
+								"smarturl",
+								"smarturl",
 								FALSE,
 								G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
 					 g_param_spec_string  ("icon",
-							       "Icon",
-							       "Icon",
+							       "icon",
+							       "icon",
 							       NULL,
 							       G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (object_class, sizeof(EphyBookmarkActionPrivate));
 }
 
-static void
-ephy_bookmark_action_init (EphyBookmarkAction *action)
+GType
+ephy_bookmark_action_get_type (void)
 {
-	action->priv = EPHY_BOOKMARK_ACTION_GET_PRIVATE (action);
-	action->priv->cache_handler = 0;
-}
+	static GType type = 0;
 
-char *
-ephy_bookmark_action_name (EphyNode *node)
-{
-	return g_strdup_printf("Bmk%u", ephy_node_get_id (node));
+	if (G_UNLIKELY (type == 0))
+	{
+		static const GTypeInfo type_info =
+		{
+			sizeof (EphyBookmarkActionClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) ephy_bookmark_action_class_init,
+			(GClassFinalizeFunc) NULL,
+			NULL,
+			sizeof (EphyBookmarkAction),
+			0, /* n_preallocs */
+			(GInstanceInitFunc) ephy_bookmark_action_init,
+		};
+
+		type = g_type_register_static (EPHY_TYPE_LINK_ACTION,
+					       "EphyBookmarkAction",
+					       &type_info, 0);
+	}
+
+	return type;
 }
 
 GtkAction *
-ephy_bookmark_action_new (EphyNode *node, char *name)
+ephy_bookmark_action_new (EphyNode *node,
+			  const char *name)
 {
-	GtkAction *action;
-	
-	if(!name) name = ephy_bookmark_action_name (node);
-	g_return_val_if_fail (name, NULL);
-	
-	action =  GTK_ACTION (g_object_new (EPHY_TYPE_BOOKMARK_ACTION,
-					    "name", name,
-					    "bookmark", node,
-					    NULL));
+	g_assert (name != NULL);
 
-	return action;
+	return  GTK_ACTION (g_object_new (EPHY_TYPE_BOOKMARK_ACTION,
+					  "name", name,
+					  "bookmark", node,
+					  NULL));
 }
