@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2000-2004 Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2003, 2004, 2006 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id$
+ *  $Id: ephy-shell.c,v 1.140 2006/02/04 10:47:27 paharvey Exp $
  */
 
 #include "config.h"
@@ -30,6 +30,7 @@
 #include "ephy-file-helpers.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-window.h"
+#include "ephy-bookmarks-ui.h"
 #include "ephy-bookmarks-import.h"
 #include "ephy-bookmarks-editor.h"
 #include "ephy-history-window.h"
@@ -38,43 +39,38 @@
 #include "ephy-debug.h"
 #include "ephy-extensions-manager.h"
 #include "ephy-session.h"
+#include "ephy-lockdown.h"
 #include "downloader-view.h"
 #include "egg-toolbars-model.h"
 #include "ephy-toolbars-model.h"
 #include "ephy-toolbar.h"
-#include "ephy-automation.h"
 #include "print-dialog.h"
 #include "ephy-prefs.h"
 #include "ephy-gui.h"
 
-#ifdef ENABLE_DBUS
-#include "ephy-dbus.h"
-#endif
-
 #include <string.h>
-#include <bonobo/bonobo-main.h>
 #include <glib/gi18n.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtknotebook.h>
 #include <dirent.h>
 #include <unistd.h>
-#include <libgnomeui/gnome-client.h>
 
-#define AUTOMATION_IID "OAFIID:GNOME_Epiphany_Automation"
+#ifdef ENABLE_NETWORK_MANAGER
+#include "ephy-net-monitor.h"
+#endif
 
 #define EPHY_SHELL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_SHELL, EphyShellPrivate))
 
 struct _EphyShellPrivate
 {
-	BonoboGenericFactory *automation_factory;
 	EphySession *session;
+	GObject *lockdown;
 	EphyBookmarks *bookmarks;
 	EggToolbarsModel *toolbars_model;
 	EggToolbarsModel *fs_toolbars_model;
 	EphyExtensionsManager *extensions_manager;
-	GObject *dbus_service;
+#ifdef ENABLE_NETWORK_MANAGER
+	EphyNetMonitor *net_monitor;
+#endif
 	GtkWidget *bme;
 	GtkWidget *history_window;
 	GObject *pdm_dialog;
@@ -82,30 +78,18 @@ struct _EphyShellPrivate
 	GObject *print_setup_dialog;
 	GList *del_on_exit;
 
-	gboolean embed_single_connected;
+	guint embed_single_connected : 1;
 };
 
 EphyShell *ephy_shell = NULL;
 
 static void ephy_shell_class_init	(EphyShellClass *klass);
 static void ephy_shell_init		(EphyShell *shell);
+static void ephy_shell_dispose		(GObject *object);
 static void ephy_shell_finalize		(GObject *object);
 static GObject *impl_get_embed_single   (EphyEmbedShell *embed_shell);
 
-static GObjectClass *parent_class = NULL;
-
-GQuark
-ephy_shell_error_quark (void)
-{
-	static GQuark q = 0;
-
-	if (q == 0)
-	{
-		q = g_quark_from_static_string ("ephy-shell-error-quark");
-	}
-
-	return q;
-}
+static GObjectClass *parent_class;
 
 GType
 ephy_shell_get_type (void)
@@ -143,6 +127,7 @@ ephy_shell_class_init (EphyShellClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	object_class->dispose = ephy_shell_dispose;
 	object_class->finalize = ephy_shell_finalize;
 
 	embed_shell_class->get_embed_single = impl_get_embed_single;
@@ -158,6 +143,7 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
 {
 	GtkWidget *parent = NULL;
 	EphyTab *new_tab;
+	gboolean is_popup;
 	EphyNewTabFlags flags = EPHY_NEW_TAB_DONT_SHOW_WINDOW |
 				EPHY_NEW_TAB_APPEND_LAST |
 				EPHY_NEW_TAB_IN_NEW_WINDOW |
@@ -179,10 +165,15 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
 		parent = gtk_widget_get_toplevel (GTK_WIDGET (parent_embed));
 	}
 
+	/* what's a popup ? ATM, any window opened with menubar toggled on 
+   	 * is *not* a popup 
+	 */
+	is_popup = (chromemask & EPHY_EMBED_CHROME_MENUBAR) == 0;
+
 	new_tab = ephy_shell_new_tab_full
 		(shell,
 		 EPHY_IS_WINDOW (parent) ? EPHY_WINDOW (parent) : NULL,
-		 NULL, NULL, flags, chromemask, 0);
+		 NULL, NULL, flags, chromemask, is_popup, 0);
 
 	return ephy_tab_get_embed (new_tab);
 }
@@ -224,15 +215,38 @@ ephy_shell_add_sidebar_cb (EphyEmbedSingle *embed_single,
 	return TRUE;
 }
 
+#ifdef ENABLE_NETWORK_MANAGER
+
+static void
+ephy_shell_sync_network_status (EphyNetMonitor *net_monitor,
+				GParamSpec *pspec,
+				EphyShell *shell)
+{
+	EphyShellPrivate *priv = shell->priv;
+	EphyEmbedSingle *single;
+	gboolean net_status;
+
+	if (!priv->embed_single_connected) return;
+
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (shell)));
+
+	net_status = ephy_net_monitor_get_net_status (net_monitor);
+	ephy_embed_single_set_network_status (single, net_status);
+}
+
+#endif /* ENABLE_NETWORK_MANAGER */
+
 static GObject*
 impl_get_embed_single (EphyEmbedShell *embed_shell)
 {
 	EphyShell *shell = EPHY_SHELL (embed_shell);
+	EphyShellPrivate *priv = shell->priv;
 	GObject *embed_single;
 
 	embed_single = EPHY_EMBED_SHELL_CLASS (parent_class)->get_embed_single (embed_shell);
 
-	if (embed_single != NULL && shell->priv->embed_single_connected == FALSE)
+	if (embed_single != NULL &&
+	    priv->embed_single_connected == FALSE)
 	{
 		g_signal_connect_object (embed_single, "new-window",
 					 G_CALLBACK (ephy_shell_new_window_cb),
@@ -242,44 +256,18 @@ impl_get_embed_single (EphyEmbedShell *embed_shell)
 					 G_CALLBACK (ephy_shell_add_sidebar_cb),
 					 shell, G_CONNECT_AFTER);
 
-		shell->priv->embed_single_connected = TRUE;
+		priv->embed_single_connected = TRUE;
+
+#ifdef ENABLE_NETWORK_MANAGER
+		/* Now we need the net monitor */
+		ephy_shell_get_net_monitor (shell);
+		ephy_shell_sync_network_status (priv->net_monitor, NULL, shell);
+#endif
 	}
 	
 	return embed_single;
 }
 
-static BonoboObject *
-ephy_automation_factory_cb (BonoboGenericFactory *this_factory,
-			    const char *iid,
-			    EphyShell *shell)
-{
-	if (strcmp (iid, AUTOMATION_IID) == 0)
-	{
-		return BONOBO_OBJECT (g_object_new (EPHY_TYPE_AUTOMATION, NULL));
-	}
-
-	g_assert_not_reached ();
-
-	return NULL;
-}
-
-static BonoboGenericFactory *
-ephy_automation_factory_new (EphyShell *shell)
-{
-	BonoboGenericFactory *factory;
-	GClosure *factory_closure;
-
-	factory = g_object_new (bonobo_generic_factory_get_type (), NULL);
-
-	factory_closure = g_cclosure_new
-		(G_CALLBACK (ephy_automation_factory_cb), shell, NULL);
-
-	bonobo_generic_factory_construct_noreg
-		(factory, AUTOMATION_FACTORY_IID, factory_closure);
-
-	return factory;
-}
-
 static void
 ephy_shell_init (EphyShell *shell)
 {
@@ -292,320 +280,116 @@ ephy_shell_init (EphyShell *shell)
 	ephy_shell = shell;
 	g_object_add_weak_pointer (G_OBJECT(ephy_shell),
 				   (gpointer *)ptr);
-
-	/* Instantiate the automation factory */
-	shell->priv->automation_factory = ephy_automation_factory_new (shell);
-}
-
-static char *
-path_from_command_line_arg (const char *arg)
-{
-	char path[PATH_MAX];
-
-	if (realpath (arg, path) != NULL)
-	{
-		return g_strdup (path);
-	}
-	else
-	{
-		return g_strdup (arg);
-	}
 }
 
 static void
-open_urls (GNOME_EphyAutomation automation,
-	   guint32 user_time,
-	   const char **args,
-	   CORBA_Environment *ev,
-	   gboolean new_tab,
-	   gboolean existing_window,
-	   gboolean fullscreen)
+ephy_shell_dispose (GObject *object)
 {
-	int i;
+	EphyShell *shell = EPHY_SHELL (object);
+	EphyShellPrivate *priv = shell->priv;
 
-	if (args == NULL)
+	LOG ("EphyShell disposing");
+
+	if (shell->priv->extensions_manager != NULL)
 	{
-		/* Homepage or resume */
-		GNOME_EphyAutomation_loadUrlWithStartupId
-			(automation, "", fullscreen,
-			 existing_window, new_tab, user_time, ev);
-	}
-	else
-	{
-		for (i = 0; args[i] != NULL; i++)
-		{
-			char *path;
-
-			path = path_from_command_line_arg (args[i]);
-
-			GNOME_EphyAutomation_loadUrlWithStartupId
-				(automation, path, fullscreen,
-				 existing_window, new_tab, user_time, ev);
-
-			g_free (path);
-		}
-	}
-}
-
-static gboolean
-save_yourself_cb (GnomeClient *client,
-		  gint phase,
-		  GnomeSaveStyle save_style,
-		  gboolean shutdown,
-		  GnomeInteractStyle interact_style,
-		  gboolean fast,
-		  EphyShell *shell)
-{
-	char *argv[] = { NULL, "--load-session", NULL };
-	char *discard_argv[] = { "rm", "-f", NULL };
-	EphySession *session;
-	char *tmp, *save_to;
-
-	LOG ("save_yourself_cb");
-
-	tmp = g_build_filename (ephy_dot_dir (),
-				"session_gnome-XXXXXX",
-				NULL);
-	save_to = ephy_file_tmp_filename (tmp, "xml");
-	g_free (tmp);
-
-	session = EPHY_SESSION (ephy_shell_get_session (shell));
-
-	argv[0] = g_get_prgname ();
-	argv[2] = save_to;
-	gnome_client_set_restart_command
-		(client, 3, argv);
-
-	discard_argv[2] = save_to;
-	gnome_client_set_discard_command (client, 3,
-					  discard_argv);
-
-	ephy_session_save (session, save_to);
-
-	g_free (save_to);
-
-	return TRUE;
-}
-
-static void
-die_cb (GnomeClient* client,
-	EphyShell *shell)
-{
-	EphySession *session;
-
-	LOG ("die_cb");
-
-	session = EPHY_SESSION (ephy_shell_get_session (shell));
-	ephy_session_close (session);
-}
-
-static void
-gnome_session_init (EphyShell *shell)
-{
-	GnomeClient *client;
-
-	client = gnome_master_client ();
-
-	g_signal_connect (G_OBJECT (client),
-			  "save_yourself",
-			  G_CALLBACK (save_yourself_cb),
-			  shell);
-	g_signal_connect (G_OBJECT (client),
-			  "die",
-			  G_CALLBACK (die_cb),
-			  shell);
-}
-
-gboolean
-ephy_shell_startup (EphyShell *shell,
-		    EphyShellStartupFlags flags,
-		    guint32 user_time,
-		    const char **args,
-		    const char *string_arg,
-		    GError **error)
-{
-	CORBA_Environment ev;
-	GNOME_EphyAutomation automation;
-	Bonobo_RegistrationResult result;
-
-	ephy_ensure_dir_exists (ephy_dot_dir ());
-
-	CORBA_exception_init (&ev);
-
-	result = bonobo_activation_register_active_server
-		(AUTOMATION_FACTORY_IID, BONOBO_OBJREF (shell->priv->automation_factory), NULL);
-
-	switch (result)
-	{
-		case Bonobo_ACTIVATION_REG_SUCCESS:
-			break;
-		case Bonobo_ACTIVATION_REG_ALREADY_ACTIVE:
-			break;
-		case Bonobo_ACTIVATION_REG_NOT_LISTED:
-			g_set_error (error, EPHY_SHELL_ERROR,
-				     EPHY_SHELL_ERROR_MISSING_SERVER,
-				     _("Bonobo couldn't locate the GNOME_Epiphany_Automation.server "
-				       "file. You can use bonobo-activation-sysconf to configure "
-				       "the search path for bonobo server files."));
-			break;
-		case Bonobo_ACTIVATION_REG_ERROR:
-			g_set_error (error, EPHY_SHELL_ERROR,
-				     EPHY_SHELL_ERROR_FACTORY_REG_FAILED,
-				     _("Epiphany can't be used now, due to an unexpected error "
-				       "from Bonobo when attempting to register the automation "
-				       "server"));
-			break;
-		default:
-			g_assert_not_reached ();
+		LOG ("Unref extension manager");
+		/* this will unload the extensions */
+		g_object_unref (priv->extensions_manager);
+		priv->extensions_manager = NULL;
 	}
 
-	if (result == Bonobo_ACTIVATION_REG_SUCCESS ||
-		 result == Bonobo_ACTIVATION_REG_ALREADY_ACTIVE)
+	if (priv->session != NULL)
 	{
-		automation = bonobo_activation_activate_from_id (AUTOMATION_IID,
-								 0, NULL, &ev);
-		if (CORBA_Object_is_nil (automation, &ev))
-		{
-			g_set_error (error, EPHY_SHELL_ERROR,
-				     EPHY_SHELL_ERROR_OBJECT_REG_FAILED,
-				     _("Epiphany can't be used now, due to an unexpected error "
-				       "from Bonobo when attempting to locate the automation "
-				       "object."));
-			automation = NULL;
-			goto done;
-		}
-
-		/* init the session manager up here so we can quit while the resume dialogue is on */
-		gnome_session_init (shell);
-
-		if (flags & EPHY_SHELL_STARTUP_BOOKMARKS_EDITOR)
-		{
-			GNOME_EphyAutomation_openBookmarksEditorWithStartupId
-				(automation, user_time, &ev);
-		}
-		else if (flags & EPHY_SHELL_STARTUP_SESSION)
-		{
-			GNOME_EphyAutomation_loadSessionWithStartupId
-				(automation, string_arg, user_time, &ev);
-		}
-		else if (flags & EPHY_SHELL_STARTUP_IMPORT_BOOKMARKS)
-		{
-			GNOME_EphyAutomation_importBookmarks
-				(automation, string_arg, &ev);
-		}
-		else if (flags & EPHY_SHELL_STARTUP_ADD_BOOKMARK)
-		{
-			GNOME_EphyAutomation_addBookmark
-				(automation, string_arg, &ev);
-		}
-		else
-		{
-			open_urls (automation, user_time, args, &ev,
-				   flags & EPHY_SHELL_STARTUP_TABS,
-				   flags & EPHY_SHELL_STARTUP_EXISTING_WINDOW,
-				   flags & EPHY_SHELL_STARTUP_FULLSCREEN);
-		}
-
-		bonobo_object_release_unref (automation, &ev);
+		LOG ("Unref session manager");
+		g_object_unref (priv->session);
+		priv->session = NULL;
 	}
 
-done:
-	CORBA_exception_free (&ev);
-	gdk_notify_startup_complete ();
+	if (priv->lockdown != NULL)
+	{
+		LOG ("Unref lockdown controller");
+		g_object_unref (priv->lockdown);
+		priv->lockdown = NULL;
+	}
 
-	return !(result == Bonobo_ACTIVATION_REG_ALREADY_ACTIVE);
+	if (priv->toolbars_model != NULL)
+	{
+		LOG ("Unref toolbars model");
+		g_object_unref (priv->toolbars_model);
+		priv->toolbars_model = NULL;
+	}
+
+	if (priv->fs_toolbars_model != NULL)
+	{
+		LOG ("Unref fullscreen toolbars model");
+		g_object_unref (priv->fs_toolbars_model);
+		priv->fs_toolbars_model = NULL;
+	}
+
+	if (priv->bme != NULL)
+	{
+		LOG ("Unref Bookmarks Editor");
+		gtk_widget_destroy (GTK_WIDGET (priv->bme));
+		priv->bme = NULL;
+	}
+
+	if (priv->history_window != NULL)
+	{
+		LOG ("Unref History Window");
+		gtk_widget_destroy (GTK_WIDGET (priv->history_window));
+		priv->history_window = NULL;
+	}
+
+	if (priv->pdm_dialog != NULL)
+	{
+		LOG ("Unref PDM Dialog");
+		g_object_unref (priv->pdm_dialog);
+		priv->pdm_dialog = NULL;
+	}
+
+	if (priv->prefs_dialog != NULL)
+	{
+		LOG ("Unref prefs dialog");
+		g_object_unref (priv->prefs_dialog);
+		priv->prefs_dialog = NULL;
+	}
+
+	if (priv->print_setup_dialog != NULL)
+	{
+		LOG ("Unref print setup dialog");
+		g_object_unref (priv->print_setup_dialog);
+		priv->print_setup_dialog = NULL;
+	}
+
+	if (priv->bookmarks != NULL)
+	{
+		LOG ("Unref bookmarks");
+		g_object_unref (priv->bookmarks);
+		priv->bookmarks = NULL;
+	}
+
+#ifdef ENABLE_NETWORK_MANAGER
+	if (priv->net_monitor != NULL)
+	{
+		LOG ("Unref net monitor");
+		g_signal_handlers_disconnect_by_func
+			(priv->net_monitor, G_CALLBACK (ephy_shell_sync_network_status), shell);
+		g_object_unref (priv->net_monitor);
+		priv->net_monitor = NULL;
+	}
+#endif /* ENABLE_NETWORK_MANAGER */
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
 ephy_shell_finalize (GObject *object)
 {
-	EphyShell *shell = EPHY_SHELL (object);
-
-	g_assert (ephy_shell == NULL);
-
-	/* this will unload the extensions */
-	LOG ("Unref extension manager");
-	if (shell->priv->extensions_manager)
-	{
-		g_object_unref (shell->priv->extensions_manager);
-	}
-
-#ifdef ENABLE_DBUS
-	LOG ("Shutting down DBUS service");
-	if (shell->priv->dbus_service)
-	{
-		g_object_unref (shell->priv->dbus_service);
-	}
-#endif
-
-	LOG ("Unref session manager");
-	if (shell->priv->session)
-	{
-		g_object_unref (shell->priv->session);
-	}
-
-	LOG ("Unref toolbars model");
-	if (shell->priv->toolbars_model)
-	{
-		g_object_unref (shell->priv->toolbars_model);
-	}
-
-	LOG ("Unref fullscreen toolbars model");
-	if (shell->priv->fs_toolbars_model)
-	{
-		g_object_unref (shell->priv->fs_toolbars_model);
-	}
-
-	LOG ("Unref Bookmarks Editor");
-	if (shell->priv->bme)
-	{
-		gtk_widget_destroy (GTK_WIDGET (shell->priv->bme));
-	}
-
-	LOG ("Unref History Window");
-	if (shell->priv->history_window)
-	{
-		gtk_widget_destroy (GTK_WIDGET (shell->priv->history_window));
-	}
-
-	LOG ("Unref PDM Dialog");
-	if (shell->priv->pdm_dialog)
-	{
-		g_object_unref (shell->priv->pdm_dialog);
-	}
-
-	LOG ("Unref prefs dialog");
-	if (shell->priv->prefs_dialog)
-	{
-		g_object_unref (shell->priv->prefs_dialog);
-	}
-
-	LOG ("Unref print setup dialog");
-	if (shell->priv->print_setup_dialog)
-	{
-		g_object_unref (shell->priv->print_setup_dialog);
-	}
-
-	LOG ("Unref bookmarks");
-	if (shell->priv->bookmarks)
-	{
-		g_object_unref (shell->priv->bookmarks);
-	}
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 
-	if (shell->priv->automation_factory)
-	{
-		bonobo_activation_unregister_active_server
-			(AUTOMATION_FACTORY_IID, BONOBO_OBJREF (shell->priv->automation_factory));
-
-		bonobo_object_unref (shell->priv->automation_factory);
-	}
-
-	LOG ("Ephy shell finalized");
+	LOG ("Ephy shell finalised");
 }
 
-
 /**
  * ephy_shell_get_default:
  *
@@ -616,19 +400,28 @@ ephy_shell_finalize (GObject *object)
 EphyShell *
 ephy_shell_get_default (void)
 {
-   return ephy_shell;
+	return ephy_shell;
 }
 
-EphyShell *
-ephy_shell_new (void)
+static gboolean
+url_is_empty (const char *location)
 {
-	return EPHY_SHELL (g_object_new (EPHY_TYPE_SHELL, NULL));
+	gboolean is_empty = FALSE;
+
+        if (location == NULL || location[0] == '\0' ||
+            strcmp (location, "about:blank") == 0)
+        {
+                is_empty = TRUE;
+        }
+
+        return is_empty;
 }
 
-static void
+static gboolean
 load_homepage (EphyEmbed *embed)
 {
 	char *home;
+	gboolean is_empty;
 
 	home = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
 
@@ -639,9 +432,13 @@ load_homepage (EphyEmbed *embed)
 		home = g_strdup ("about:blank");
 	}
 
+	is_empty = url_is_empty (home);
+
 	ephy_embed_load_url (embed, home);
 
 	g_free (home);
+
+	return is_empty;
 }
 
 /**
@@ -651,6 +448,7 @@ load_homepage (EphyEmbed *embed)
  * @previous_tab: the referrer tab or %NULL
  * @url: an url to load or %NULL
  * @chrome: a #EphyEmbedChrome mask to use if creating a new window
+ * @is_popup: whether the new window is a popup
  * @user_time: a timestamp, or 0
  *
  * Create a new tab and the parent window when necessary.
@@ -665,6 +463,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
 			 const char *url,
 			 EphyNewTabFlags flags,
 			 EphyEmbedChrome chrome,
+			 gboolean is_popup,
 			 guint32 user_time)
 {
 	EphyWindow *window;
@@ -675,12 +474,14 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	EphyEmbed *previous_embed = NULL;
 	GtkWidget *nb;
 	int position = -1;
+	gboolean is_empty = FALSE;
 	EphyToolbar *toolbar;
 
 	if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
 	if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
 
 	in_new_window = in_new_window && !eel_gconf_get_boolean (CONF_LOCKDOWN_FULLSCREEN);
+	g_return_val_if_fail (in_new_window || !is_popup, NULL);
 
 	jump_to = (flags & EPHY_NEW_TAB_JUMP) != 0;
 
@@ -693,7 +494,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	}
 	else
 	{
-		window = ephy_window_new_with_chrome (chrome);
+		window = ephy_window_new_with_chrome (chrome, is_popup);
 	}
 
 	toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
@@ -731,16 +532,39 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
 	    flags & EPHY_NEW_TAB_NEW_PAGE)
 	{
-		ephy_tab_set_location (tab, "", EPHY_TAB_ADDRESS_EXPIRE_NEXT);
+		ephy_tab_set_typed_address (tab, "",
+					    EPHY_TAB_ADDRESS_EXPIRE_NEXT);
 		ephy_toolbar_activate_location (toolbar);
-		load_homepage (embed);
+		is_empty = load_homepage (embed);
 	}
 	else if (flags & EPHY_NEW_TAB_OPEN_PAGE)
 	{
 		g_assert (url != NULL);
 		ephy_embed_load_url (embed, url);
+		is_empty = url_is_empty (url);
 	}
 
+        /* Make sure the initial focus is somewhere sensible and not, for
+         * example, on the reload button.
+         */
+        if (in_new_window || jump_to)
+        {
+                /* If the location entry is blank, focus that, except if the
+                 * page was a copy */
+                if (is_empty)
+                {
+                        /* empty page, focus location entry */
+			toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
+			ephy_toolbar_activate_location (toolbar);
+                }
+                else if (embed != NULL) 
+                {
+			/* non-empty page, focus the page. but make sure the widget is realised first! */
+			gtk_widget_realize (GTK_WIDGET (embed));
+			gtk_widget_grab_focus (GTK_WIDGET (embed));
+                }
+        }
+
 	return tab;
 }
 
@@ -765,7 +589,7 @@ ephy_shell_new_tab (EphyShell *shell,
 {
 	return ephy_shell_new_tab_full (shell, parent_window,
 					previous_tab, url, flags,
-					EPHY_EMBED_CHROME_ALL, 0);
+					EPHY_EMBED_CHROME_ALL, FALSE, 0);
 }
 
 /**
@@ -796,6 +620,34 @@ ephy_shell_get_session (EphyShell *shell)
 	return G_OBJECT (shell->priv->session);
 }
 
+/**
+ * ephy_shell_get_lockdown:
+ * @shell: the #EphyShell
+ *
+ * Returns the lockdown controller.
+ *
+ * Return value: the lockdown controller
+ **/
+static GObject *
+ephy_shell_get_lockdown (EphyShell *shell)
+{
+	g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
+
+	if (shell->priv->lockdown == NULL)
+	{
+		EphyExtensionsManager *manager;
+
+		shell->priv->lockdown = g_object_new (EPHY_TYPE_LOCKDOWN, NULL);
+
+		manager = EPHY_EXTENSIONS_MANAGER
+			(ephy_shell_get_extensions_manager (shell));
+		ephy_extensions_manager_register (manager,
+						  G_OBJECT (shell->priv->lockdown));
+	}
+
+	return G_OBJECT (shell->priv->session);
+}
+
 EphyBookmarks *
 ephy_shell_get_bookmarks (EphyShell *shell)
 {
@@ -823,7 +675,7 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 			xml = ephy_file ("epiphany-fs-toolbar.xml");
 			g_return_val_if_fail (xml != NULL, NULL);
 
-			success = egg_toolbars_model_load
+			success = egg_toolbars_model_load_toolbars
 				(shell->priv->fs_toolbars_model, xml);
 			g_return_val_if_fail (success, NULL);
 		}
@@ -834,19 +686,10 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 	{
 		if (shell->priv->toolbars_model == NULL)
 		{
-			EphyBookmarks *bookmarks;
-			GObject *bookmarksbar_model;
-
 			shell->priv->toolbars_model = ephy_toolbars_model_new ();
-
-			/* get the bookmarks toolbars model. we have to do this
-			 * before loading the toolbars model from disk, since
-			 * this will connect the get_item_* signals
-			 */
-			bookmarks = ephy_shell_get_bookmarks (shell);
-			bookmarksbar_model = ephy_bookmarks_get_toolbars_model (bookmarks);
-
-			/* ok, now we can load the model */
+			
+			ephy_bookmarks_ui_attach_toolbar_model (shell->priv->toolbars_model);
+			
 			ephy_toolbars_model_load
 				(EPHY_TOOLBARS_MODEL (shell->priv->toolbars_model));
 		}
@@ -867,11 +710,34 @@ ephy_shell_get_extensions_manager (EphyShell *es)
 			g_object_new (EPHY_TYPE_EXTENSIONS_MANAGER, NULL);
 
 		ephy_extensions_manager_startup (es->priv->extensions_manager);
+
+		/* FIXME */
+		ephy_shell_get_lockdown (es);
+		ephy_embed_shell_get_adblock_manager (embed_shell);
 	}
 
 	return G_OBJECT (es->priv->extensions_manager);
 }
 
+GObject *
+ephy_shell_get_net_monitor (EphyShell *shell)
+{
+#ifdef ENABLE_NETWORK_MANAGER
+	EphyShellPrivate *priv = shell->priv;
+
+	if (priv->net_monitor == NULL)
+	{
+		priv->net_monitor = ephy_net_monitor_new ();
+		g_signal_connect (priv->net_monitor, "notify::network-status",
+				  G_CALLBACK (ephy_shell_sync_network_status), shell);
+	}
+
+	return G_OBJECT (priv->net_monitor);
+#else
+	return NULL;
+#endif /* ENABLE_NETWORK_MANAGER */
+}
+
 static void
 toolwindow_show_cb (GtkWidget *widget, EphyShell *es)
 {
@@ -979,18 +845,13 @@ ephy_shell_get_print_setup_dialog (EphyShell *shell)
 	return shell->priv->print_setup_dialog;
 }
 
-#ifdef ENABLE_DBUS
-GObject	*
-ephy_shell_get_dbus_service (EphyShell *shell)
+void
+_ephy_shell_create_instance (void)
 {
-	g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
+	g_assert (ephy_shell == NULL);
 
-	if (shell->priv->dbus_service == NULL)
-	{
-		shell->priv->dbus_service = g_object_new (EPHY_TYPE_DBUS, NULL);
-		ephy_dbus_startup (EPHY_DBUS (shell->priv->dbus_service));
-	}
+	ephy_shell = EPHY_SHELL (g_object_new (EPHY_TYPE_SHELL, NULL));
+	/* FIXME weak ref */
 
-	return G_OBJECT (shell->priv->dbus_service);
+	g_assert (ephy_shell != NULL);
 }
-#endif
